// ignore_for_file: avoid_print

import 'dart:convert';
import 'dart:io';

import 'package:pubspec_yaml/pubspec_yaml.dart';
import 'package:translaty_fetcher/translaty_fetcher.dart';
import 'package:yaml/yaml.dart';

Future<void> translatyGenerate(String sourceDir) async {
  final stopwatch = Stopwatch()..start();
  final file = File('translaty.yaml');
  if (!(await file.exists())) {
    print("ERROR - translaty.yaml file does not exist.");
    throw Exception();
  }
  final yaml = await file.readAsString().then(
        (value) => loadYaml(value),
      );

  final translatyConfig = yaml["translaty"];

  if (translatyConfig == null) {
    print(
      "ERROR - Did not find the point translaty in the pubspec.yaml file. Aborting",
    );
    throw Exception();
  }
  final clientKey = translatyConfig["client_key"];
  if (clientKey == null) {
    print(
      "ERROR - The translaty-section in your pubspec.yaml does not contain a client_key. Aborting",
    );
    throw Exception();
  }
  final String? translatyUrl = translatyConfig["translaty_url"];

  final dataFuture = TranslatyFetcher().getAllTranslations(
    translatyClientKey: clientKey,
    translatyUrl: translatyUrl,
  );

  final targetDir = Directory("./$sourceDir");
  await targetDir.exists().then(
    (doesExist) async {
      if (!doesExist) {
        await targetDir.create(recursive: true);
      }
    },
  );

  final data = await dataFuture;
  _prepareData(data);
  print("Found ${data.length} languages. Creating files.");

  var encoder = const JsonEncoder.withIndent("     ");
  for (final item in data.entries) {
    final language = item.key;
    final targetPath = "${targetDir.path}/${_getFileName(language)}.json";
    final jsonData = encoder.convert(item.value);
    await File(targetPath).writeAsString(jsonData);
  }
  stopwatch.stop();
  print(
    "Finished to write json files which took ${stopwatch.elapsed.inMilliseconds}ms, starting code generation.",
  );

  await _checkAssetsPathInPubspec(sourceDir);
}

String _getFileName(String language) {
  if (language.length == 5) {
    final lan = language.substring(0, 2).toLowerCase();
    final country = language.substring(3, 5).toUpperCase();
    return "$lan-$country";
  } else {
    return language.toLowerCase();
  }
}

Future<void> _checkAssetsPathInPubspec(String sourceDir) async {
  final pubspecYaml = await File('pubspec.yaml').readAsString().then(
        (value) => value.toPubspecYaml(),
      );
  final assets = pubspecYaml.customFields["flutter"]["assets"];
  final requiredKeys = [
    sourceDir.endsWith("/") ? sourceDir : "$sourceDir/",
    "translaty.yaml",
  ];

  if (assets == null) {
    print(
      "WARNING - We did not find the assets-section in your pubspec.yaml. Is it missing?",
    );
    return;
  }
  final assetList = assets as List<dynamic>;
  for (final requiredKey in requiredKeys) {
    if (!assetList.contains(requiredKey)) {
      print(
        "WARNING - We did not find $requiredKey listed as an asset in your pubspec.yaml. This is required for the translations to work!",
      );
    }
  }
}

/// Fills up nulls
void _prepareData(Map<String, Map<String, dynamic>> data) {
  final uniqueKeys = _getAllDistinctKeys(data);
  for (final item in data.values) {
    final uniqueKeysCopy = Set.of(uniqueKeys);
    uniqueKeysCopy.removeAll(item.keys);

    for (final missingKey in uniqueKeysCopy) {
      item[missingKey] = null;
    }
  }
}

Set<String> _getAllDistinctKeys(Map<String, Map<String, dynamic>> data) {
  final all = <String>{};
  for (final item in data.values) {
    all.addAll(item.keys);
  }
  return all;
}
